@@ -36,10 +36,11 @@
 // 
 // ***** END LICENSE BLOCK *****
 
-var EXPORTED_SYMBOLS = ["Server", "server", "AsyncRead", "Session", "sessions", "globalRegistry"];
+var EXPORTED_SYMBOLS = ["Server", "server", "AsyncRead", "Session", "sessions", "globalRegistry", "startServer"];
 
 var events = {}; Components.utils.import("resource://jsbridge/modules/events.js", events);
-
+const DEBUG_ON = false;
+const BUFFER_SIZE = 1024;
 const Cc = Components.classes;
 const Ci = Components.interfaces;
 const loader = Cc['@mozilla.org/moz/jssubscript-loader;1']
@@ -49,9 +50,6 @@ var hwindow = Components.classes["@mozilla.org/appshell/appShellService;1"]
     .getService(Components.interfaces.nsIAppShellService)
     .hiddenDOMWindow;
     
-var nativeJSON = Components.classes["@mozilla.org/dom/json;1"]
-    .createInstance(Components.interfaces.nsIJSON);
-
 var json2 = Components.utils.import("resource://jsbridge/modules/json2.js");
 
 var jsonEncode = json2.JSON.stringify;    
@@ -64,11 +62,29 @@ function AsyncRead (session) {
 }
 AsyncRead.prototype.onStartRequest = function (request, context) {};
 AsyncRead.prototype.onStopRequest = function (request, context, status) {
+  log("async onstoprequest: onstoprequest");
   this.session.onQuit();
 }
 AsyncRead.prototype.onDataAvailable = function (request, context, inputStream, offset, count) {
   var str = {};
-  this.session.instream.readString(count, str);
+  str.value = '';
+
+  var bytesAvail = 0;
+  do {
+    var parts = {};
+    if (count > BUFFER_SIZE) {
+      bytesAvail = BUFFER_SIZE;
+    } else {
+      bytesAvail = count;
+    }
+
+    log("jsbridge: onDataAvailable, reading bytesAvail = " + bytesAvail + "\n");
+    var bytesRead = this.session.instream.readString(bytesAvail, parts);
+    count = count - bytesRead;
+    log("jsbridge: onDataAvailable, read bytes: " + bytesRead + " count is now: " + count + "\n"); 
+    str.value += parts.value;
+  } while (count > 0);
+  log("jsbridge: onDataAvailable, going into receive with: \n\n" + str.value + "\n\n");
   this.session.receive(str.value);
 }
 
@@ -91,7 +107,12 @@ Bridge.prototype.register = function (uuid, _type) {
     this._register(_type);
     var passed = true;
   } catch(e) {
-    this.session.encodeOut({'result':false, 'exception':{'name':e.name, 'message':e.message}, 'uuid':uuid});
+    if (typeof(e) == "string") {
+      var exception = e;
+    } else {
+      var exception = {'name':e.name, 'message':e.message};
+    }
+    this.session.encodeOut({'result':false, 'exception':exception, 'uuid':uuid});
   }
   if (passed != undefined) {
     this.session.encodeOut({"result":true, 'eventType':'register', 'uuid':uuid});
@@ -114,7 +135,7 @@ Bridge.prototype._describe = function (obj) {
       response.attributes = response.attributes.concat(i);
     }
   }
-  else {
+  else if (type != "function"){
     response.data = obj;
   }
   response.type = type;
@@ -140,11 +161,15 @@ Bridge.prototype._setAttribute = function (obj, name, value) {
   return value;
 }
 Bridge.prototype.setAttribute = function (uuid, obj, name, value) {
-  // log(uuid, String(obj), name, String(value))
   try {
     var result = this._setAttribute(obj, name, value);
   } catch(e) {
-    this.session.encodeOut({'result':false, 'exception':{'name':e.name, 'message':e.message}, 'uuid':uuid});
+    if (typeof(e) == "string") {
+      var exception = e;
+    } else {
+      var exception = {'name':e.name, 'message':e.message};
+    }
+    this.session.encodeOut({'result':false, 'exception':exception, 'uuid':uuid});
   }
   if (result != undefined) {
     this.set(uuid, obj[name]);
@@ -158,13 +183,21 @@ Bridge.prototype.execFunction = function (uuid, func, args) {
     var data = this._execFunction(func, args);
     var result = true;
   } catch(e) {
-    this.session.encodeOut({'result':false, 'exception':{'name':e.name, 'message':e.message}, 'uuid':uuid});
+    if (typeof(e) == "string") {
+      var exception = e;
+    } else {
+      var exception = {'name':e.name, 'message':e.message};
+    }
+    this.session.encodeOut({'result':false, 'exception':exception, 'uuid':uuid});
     var result = true;
   }  
   if (data != undefined) {
     this.set(uuid, data);
   } else if ( result == true) {
     this.session.encodeOut({'result':true, 'data':null, 'uuid':uuid});
+  } else {
+    log("jsbridge threw unknown data in execFunc");
+    throw 'JSBridge unknown data in execFunc';
   }
 }
 
@@ -176,16 +209,20 @@ function Session (transport) {
   this.sandbox.bridge = new Bridge(this);
   this.sandbox.openPreferences = hwindow.openPreferences;
   try {
-      this.outstream = transport.openOutputStream(Ci.nsITransport.OPEN_BLOCKING , 0, 0);
+      this.outputstream = transport.openOutputStream(Ci.nsITransport.OPEN_BLOCKING, 0, 0);	
+      this.outstream = Cc['@mozilla.org/intl/converter-output-stream;1']
+                    .createInstance(Ci.nsIConverterOutputStream);
+      this.outstream.init(this.outputstream, 'UTF-8', BUFFER_SIZE,
+                    Ci.nsIConverterOutputStream.DEFAULT_REPLACEMENT_CHARACTER);
       this.stream = transport.openInputStream(0, 0, 0);
       this.instream = Cc['@mozilla.org/intl/converter-input-stream;1']
           .createInstance(Ci.nsIConverterInputStream);
-      this.instream.init(this.stream, 'UTF-8', 1024,
+      this.instream.init(this.stream, 'UTF-8', BUFFER_SIZE,
                     Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
   } catch(e) {
       log('jsbridge: Error: ' + e);
   }
-  // log('jsbridge: Accepted connection.');
+  log('jsbridge: Accepted connection.');
   
   this.pump = Cc['@mozilla.org/network/input-stream-pump;1']
       .createInstance(Ci.nsIInputStreamPump);
@@ -193,8 +230,36 @@ function Session (transport) {
   this.pump.asyncRead(new AsyncRead(this), null);
 }
 Session.prototype.onOutput = function(string) {
-  // log('jsbridge write: '+string)
-  this.outstream.write(string, string.length);
+  log('jsbridge: write: '+string)
+  if (typeof(string) != "string") {
+    throw "This is not a string"
+  } 
+  try {
+    var stroffset = 0;
+    do {
+      var parts = '';
+      // Handle the case where we are writing something larger than our buffer
+      if (string.length > BUFFER_SIZE) {
+        log("jsbridge: onOutput: writing data stroffset is: " + stroffset + " string.length is: " + string.length);
+        parts = string.slice(stroffset, stroffset + BUFFER_SIZE);
+        log("jsbridge: onOutput: here is our part: \n" + parts + "\n");
+      } else {
+        parts = string;
+      }
+
+      // Update our offset
+      stroffset = stroffset += parts.length;
+
+      // write it
+      this.outstream.writeString(parts);
+    } while (stroffset < string.length);
+
+    // Ensure the entire stream is flushed
+    this.outstream.flush();
+  } catch (e) {
+    log("jsbridge: threw on writing string: " + string + "   exception: " + e);
+    throw "JSBridge cannot write: "+string
+  }
 };
 Session.prototype.onQuit = function() {
   this.instream.close();
@@ -205,12 +270,16 @@ Session.prototype.encodeOut = function (obj) {
   try {
     this.onOutput(jsonEncode(obj));
   } catch(e) {
-    this.onOutput(jsonEncode({'result':false, 'exception':{'name':e.name, 'message':e.message}}));
+    if (typeof(e) == "string") {
+      var exception = e;
+    } else {
+      var exception = {'name':e.name, 'message':e.message};
+    }
+    this.onOutput(jsonEncode({'result':false, 'exception':exception}));
   }
   
 }
 Session.prototype.receive = function(data) {
-  // log('jsbrige receive: '+data);
   Components.utils.evalInSandbox(data, this.sandbox);
 }
 
@@ -243,7 +312,6 @@ Server.prototype.start = function () {
         .createInstance(Ci.nsIServerSocket);
     this.serv.init(this.port, true, -1);
     this.serv.asyncListen(this);
-    // log('jsbridge: Listening...');
   } catch(e) {
     log('jsbridge: Exception: ' + e);
   }    
@@ -263,8 +331,14 @@ Server.prototype.onSocketAccepted = function (serv, transport) {
 }
 
 function log(msg) {
+  if (DEBUG_ON) {
     dump(msg + '\n');
+  } 
 }
 
-var server = new Server(24242)
-server.start()
+function startServer(port) {
+  var server = new Server(port)
+  server.start()
+}
+
+
